@@ -1,10 +1,13 @@
 import json
+import pathlib
 import inspect
 import pydantic
 from typing import Union, Type, cast
 from dataclasses import is_dataclass, asdict
 from pydantic.dataclasses import dataclass as pydantic_dataclass
 from .exceptions import ArgumentRequiredError, ArgumentError, RequestSchemaValidationError
+from .storage import Storage
+from . import envs
 
 PydanticModel = Union[Type[pydantic.BaseModel], Type["Dataclass"]]
 
@@ -77,7 +80,10 @@ class Number(BaseArgument):
         if value is None:
             return value
 
-        return float(value) if '.' in value else int(value)
+        if isinstance(value, str):
+            return float(value) if '.' in value else int(value)
+        else:
+            return value
 
 
 class Bool(BaseArgument):
@@ -155,6 +161,50 @@ class Dataclass(Json):
         return json.dumps(data)
 
 
+class File(BaseArgument):
+    FILENAME = "file"
+
+    def __init__(self, key, alias=None, required=False, default=None, arg_spec: inspect.FullArgSpec = None, **kwargs):
+        request_id = kwargs.get('request_id')
+        super().__init__(key, alias, required, default, arg_spec, **kwargs)
+        self.request_id = request_id.replace("-", "") if request_id else None
+
+    def load(self, value):
+        value = super(File, self).load(value)
+        if value is None:
+            return value
+
+        key = pathlib.PurePosixPath(value) / self.FILENAME
+        return str(key)
+
+    def save(self, value):
+        value = super(File, self).save(value)
+        if value is None:
+            return value
+
+        storage = Storage.get_instance()
+        k = pathlib.PurePosixPath('studio', envs.userId, 'tmp', envs.appId, self.request_id, envs.nodeId, self.key)
+        key = str(k / self.FILENAME)
+        storage.upload(key, value)
+        return str(k)
+
+
+class Csv(BaseArgument):
+    FILENAME = 'data.csv'
+
+
+class Excel(BaseArgument):
+    FILENAME = 'data.xlsx'
+
+
+class Npy(BaseArgument):
+    FILENAME = 'data.npy'
+
+
+class Image(BaseArgument):
+    FILENAME = 'image.png'
+
+
 DEFAULT_DATA_SUBTYPE_ARGS_MAP = {
     "all": String,
     "number": Number,
@@ -163,7 +213,7 @@ DEFAULT_DATA_SUBTYPE_ARGS_MAP = {
     "array": Json,
     "dyadicArray": Json,
     "json": Json,
-    # "file": File,
+    "file": File,
     "system": String,
     # not used now
     "int": Int,
@@ -175,7 +225,7 @@ DEFAULT_DATA_SUBTYPE_ARGS_MAP = {
 def init_arg_from_arg_spec(name, arg_spec):
     annotation = arg_spec.annotations.get(name)
     if inspect.isclass(annotation):
-        if issubclass(annotation, dict) or issubclass(annotation, list):
+        if issubclass(annotation, (dict, list)):
             return Json
         elif issubclass(annotation, int):
             return Int
@@ -186,22 +236,64 @@ def init_arg_from_arg_spec(name, arg_spec):
         elif is_dataclass(annotation) or issubclass(annotation, pydantic.BaseModel):
             return Dataclass
 
+    if isinstance(annotation, str):
+        if annotation == 'File':
+            return File
+        elif annotation == 'Csv':
+            return Csv
+        elif annotation == 'Excel':
+            return Excel
+        elif annotation == 'Npy':
+            return Npy
+        elif annotation == 'Image':
+            return Image
+
     return String
 
 
-def init_return_from_value(value):
-    if isinstance(value, dict) or isinstance(value, list):
-        return Json
-    elif isinstance(value, int):
-        return Int
-    elif isinstance(value, float):
-        return Float
-    elif isinstance(value, bool):
-        return Bool
-    elif is_dataclass(value) or isinstance(value, pydantic.BaseModel):
-        return Dataclass
+def init_return_from_value(value, arg_spec, index):
+    annotation = arg_spec.annotations.get('return')
+    if isinstance(annotation, tuple):
+        annotation = annotation[index]
+
+    if annotation:
+        if inspect.isclass(annotation):
+            if issubclass(annotation, (dict, list)):
+                return Json
+            elif issubclass(annotation, int):
+                return Int
+            elif issubclass(annotation, float):
+                return Float
+            elif issubclass(annotation, bool):
+                return Bool
+            elif is_dataclass(annotation) or issubclass(annotation, pydantic.BaseModel):
+                return Dataclass
+
+        if annotation == 'File':
+            return File
+        elif annotation == 'Csv':
+            return Csv
+        elif annotation == 'Excel':
+            return Excel
+        elif annotation == 'Npy':
+            return Npy
+        elif annotation == 'Image':
+            return Image
     else:
-        return String
+        if isinstance(value, (dict, list)):
+            return Json
+        elif isinstance(value, int):
+            return Int
+        elif isinstance(value, float):
+            return Float
+        elif isinstance(value, bool):
+            return Bool
+        elif is_dataclass(value) or isinstance(value, pydantic.BaseModel):
+            return Dataclass
+        else:
+            return String
+
+    return String
 
 
 def init_params_from_arg_spec(name, arg_spec):
